use std::rc::Rc;

use glib::subclass::InitializingObject;
use gtk::glib::object::ObjectExt;
use gtk::glib::{clone, closure_local};
use gtk::subclass::prelude::*;
use gtk::{glib, Button, CompositeTemplate, ScrolledWindow, SearchEntry, Window};
use gtk::{prelude::*, FileDialog};
use sqlite;

use crate::ui::cards::new::*;
use crate::widgets::card_entry::CardEntry;

#[derive(CompositeTemplate, Default)]
#[template(resource = "/org/foxarmy/learn-hieroglyph/cards/edit/ui.xml")]
pub struct MemoryCardsEditScene {
    #[template_child]
    pub search_entry: TemplateChild<SearchEntry>,
    #[template_child]
    pub add_button: TemplateChild<Button>,
    #[template_child]
    pub first: TemplateChild<CardEntry>,
    #[template_child]
    pub cards: TemplateChild<ScrolledWindow>,
}

#[glib::object_subclass]
impl ObjectSubclass for MemoryCardsEditScene {
    const NAME: &'static str = "MemoryCardsEditScene";
    type Type = super::MemoryCardsEditScene;
    type ParentType = gtk::ApplicationWindow;

    fn class_init(klass: &mut Self::Class) {
        klass.bind_template();
    }

    fn instance_init(obj: &InitializingObject<Self>) {
        obj.init_template();
    }
}

impl ObjectImpl for MemoryCardsEditScene {
    fn constructed(&self) {
        self.parent_constructed();

        let binding = self.obj();

        self.add_button.connect_closure("clicked", 
                                        false,
                                        closure_local!(@strong binding => move |_b: &Button| {
                                        let new_win = Rc::new(MemoryCardsNewScene::new(&binding.application().unwrap()));
                                        new_win.get_file_choose_button().connect_clicked(clone!(@strong new_win => move |_| {
                                            gtk::glib::MainContext::default().spawn_local(file_choose_dialog(Rc::clone(&new_win)));
                                        }));
                                        new_win.present();
        }));

        let c: &ScrolledWindow = self.cards.as_ref();
        c.set_child(glib::bitflags::__private::core::option::Option::Some(
            &CardEntry::new(&"a".to_string(), &"a".to_string()),
        ));
    }
}

async fn file_choose_dialog<W: IsA<gtk::Window>>(window: Rc<W>) {
    let dialog: FileDialog = gtk::FileDialog::builder().build();
    let answer = dialog.open_future(Some(&*window)).await;
    let path = answer.unwrap().path().unwrap();
    let path = path.as_path().to_str();
    let w: &MemoryCardsNewScene = Into::<&Window>::into(window.upcast_ref()).downcast_ref().unwrap(); // Weird casting from &Window as passed in func to &MemoryCardsNewScene
    w.get_image_widget().set_file(path);

    let conn = sqlite::open("test.db").unwrap();

    conn.execute("CREATE TABLE test(field_one TEXT, field_two INTEGER); INSERT INTO test VALUES ('Suka', 15);").unwrap();
}

impl WidgetImpl for MemoryCardsEditScene {}

impl WindowImpl for MemoryCardsEditScene {}

impl ApplicationWindowImpl for MemoryCardsEditScene {}
